# Copyright 2001, Jan D. Wolter, All Rights Reserved.

# deftab is our table of defines.  It's an array of arrays, with each inner
# array having the following columns
#
#  0:  name
#  1:  minmatch  -  minimum number of letters that must be given.
#  2:  maxmatch  -  length of name
#  3:  mask      -  mask value
#  4:  value     -  value
#  5:  func      -  if defined, call this with (name,mask,value) when changed

my @deftab;

# Complied in defaults for deftab in same format.
#  Flag bits are:
#    1  OK prompt macro
#    2  variable
#    4  item ranges
#    8  RFP prompt macro
#   16
#   32
#   64  things set in rc file
#  128  things set in conf rc file
#  256  things not to print when listing definitions
# 1024  things set by remote rc files
 
my @defdefault= (
    ['bullmsg',   7, 7,   2, '%(1x%3g%)%c', \&defmsg],
    ['checkmsg',  8, 8,   2, '%(1x\nnew resp items  conference\n%)%(2x%(k--%E  %)%(B>%E %)%4r %4b %s%(B  (where you currently are!)%)%)', \&defmsg],
    ['confmsg',   7, 7,   2, 'Conference: %d (%s)\nServer: %h (%H)\nParticipation file: %p\nConference type: %t (%D)\n%(i%i item%S\nFirst item %f, last %l%ENo items yet.%)', \&defmsg],
    ['fsep',      4, 4,   2, '%(Ritem %i response %r (%l)%(L:\n%)%)%(L%7N: %L%)' , \&defsep],
    ['ftsep',     5, 5,   2, 'item %i (%l) title:\n         %h' , \&defsep],
    ['indxmsg',   8, 8,   2, '%(1x%2g%)%c', \&defmsg],
    ['isep',      4, 4,   2, '\nItem %i entered %d by %a%(U uid %u%)\n %h' , \&defsep],
    ['ishort',    6, 6,   2, '%(B\nitem nresp header\n\n%)%3i %3n %h' , \&defsep],
    ['joinmsg',   7, 7,   2, '%QFile is: %d (%s)', \&defmsg],
    ['linmsg',    6, 6,   2, '%(1x%(2x\n%)%0g%)%(2x%(3N%3g%)%(n%(r%r newresponse item%S%)%(b%(r and %)%b brandnew item%S%)\n%)%(iFirst item %f, last %l\n%)%(sYou are a fair-witness in this conference.\n%)%(lYou are an observer of this conference.\n%)%(jYour name is \"%u\" in this conference.\n%(~O\n>>>> New users: type HELP for help.\n%)%)%)%c', \&defmsg],
    ['loutmsg',   7, 7,   2, '%(1x%1g%)%c', \&defmsg],
    ['mailmsg',   7, 7,   2, 'You have %(2xmore %)mail.\n%c', \&defmsg],
    ['nsep',      4, 4,   2, '\n%(N%r new of %)%zn response%S total.\n', \&defsep],
    ['partmsg',   7, 7,   2, '%(2x%10v %o %u%)%(4x\n%k participant%S total.%)%(1x\n   loginid        last time on      name\n%)', \&defmsg],
    ['printmsg',  8, 8,   2, '%QPrinted from %H conference %s on %N', \&defmsg],
    ['rsep',      4, 4,   2, '#%r %a:', \&defsep],
    ['txtsep',    6, 6,   2, '%(L%(F%7N:%)%X%L%)', \&defsep],
    ['zsep',      4, 4,   2, '%(T\f%)%c', \&defsep],
    ['all',       3, 3, 260, '*'],
    ['first',     3, 5, 268, '^'],
    ['last',      3, 4, 268, '$'],
    ['this',      3, 4, 260, '.'],
    ['next',      3, 4, 260, '>'],
    ['previous',  3, 8, 260, '<'],
    ['text',      3, 4, 264, '0'],
    ['current',   3, 7, 264, '.'],
    ['^',         1, 1, 264, '1'],
    ['prompt',    6, 6,   2, "\nOk: ", \&defprompt],
    ['noconfp',   7, 7,   2, "\n(no conf): ", \&defprompt],
    ['rfpprompt', 9, 9,   2, "\nRespond or pass? ", \&defprompt],
    ['obvprompt', 9, 9,   2, "\nResponse not possible.  Pass? ", \&defprompt],
    ['joqprompt', 9, 9,   2, "\nJoin, quit or help? ", \&defprompt],
    ['text',      4, 4,   2, '>', \&defprompt],
    ['escape',    6, 6,   2, ':', \&defprompt],
    ['gecos',     5, 5,   2, '', \&defprompt],
    ['editor',    6, 6,   2, $DEFAULT_EDITOR, \&defprompt],
    ['pager',     5, 5,   2, $DEFAULT_PAGER, \&defprompt],
    );


# defsep($name, $value)
# defmsg($name, $value)
#   These are called when system variables are redefined.  They compile the
#   new value and save the result in the %defhash hash.

sub defsep {
    my ($name, $value)= @_;
    print "DEFSET($name,$value)\n" if $debug;
    if ($name eq 'rsep')
    {
    	$value.= "%(U uid %u%)" if $value !~ /%\d*\.?\d*u/;
    	$value.= "%(D on %0d%)" if $value !~ /%\d*\.?\d*[dt]/;
    	$value.= "%(V   <censored>%)%(W   <censored & scribbled>%)"
	    if $value !~ /%\([VW]/;
    	print "rsep=$value\n" if $debug;
    }
    if ($name eq 'isep' or $name eq 'ishort')
    {
	$value.= "%(M\\n   <item is linked>%)" if $value !~ /%\(M/;
	$value.= "%(X\\n   <item is retired>%)" if $value !~ /%\(X/;
	$value.= "%(V\\n   <item is forgotten>%)" if $value !~ /%\(V/;
	$value.= "%(Z\\n   <item is frozen>%)" if $value !~ /%\(Z/;
	$value.= "%(Q\\n   <item is a favorite>%)" if $value !~ /%\(Q/;
    	$value.= "%(V\\n   <censored>%)%(W\n   <censored & scribbled>%)"
	    if $value !~ /%\([VW]/;
    	print "$name=$value\n" if $debug;
    }
    print "COMPILING $value\n" if $debug;
    $defhash{$name}= compile_sep($value, 1);
}

sub defmsg {
    my ($name, $value)= @_;
    $defhash{$name}= compile_sep($value, 0);
}

sub defprompt {
    my ($name, $value)= @_;
    $defhash{$name}= $value;
}


# init_def()
#   After we are done loading all the source files, we should compile any
#   uncompiled separators.

sub init_def {
    my ($def, $val);

    foreach $def (@defdefault)
    {
	# if we already have a value, then default was overridden
	if (defined ($val= defval($def->[0])))
	{
	    !defined $def->[5] or &{$def->[5]}($def->[0], $val);
	    next;
	}

	# use default
	push @deftab, $def;
    	!defined $def->[5] or &{$def->[5]}($def->[0], $def->[4]);
    }
}

# defval($name, $mask)
#   Return the value of the named define.  If mask is defined, return only
#   values matching the mask.

sub defval {
    my ($name, $mask)= @_;
    my ($i, $len);
    $len= length($name);

    for ($i= 0; $i < @deftab; $i++)
    {
    	if ($len >= $deftab[$i]->[1] and $len <= $deftab[$i]->[2] and
	    substr($deftab[$i]->[0],0,$len) eq $name)
	{
	    return (!$mask or ($mask & $deftab[$i]->[3])) ?
	    	$deftab[$i]->[4] : undef;
	}
    }
    return undef;
}


# new_define($name, $mask, $value)
#   Store a new value for a definition

sub new_define {
    my ($name, $mask, $value)= @_;
    my ($i, $minlen, $maxlen, $minname);

    $mask|= 64 if $cmd_source == 0;
    $mask|= 128 if $cmd_source == 3;
    $mask|= 1024 if $remote_source;

    # Remove any underscores, figuring out min and max lengths.
    $minlen= $maxlen= length($name);
    for ($i= 1; $i < $maxlen; $i++)
    {
    	if (substr($name,$i,1) eq '_')
	{
	    $minlen= $i;
	    $name=~ s/_//g;
	    $maxlen= length($name);
	    last;
	}
    }

    # Things we don't do when 'someremote' is set
    return if (flagval('remote') == 2 and $remote_source and
	       ($name eq 'editor' or $name eq 'pager' or
		(($mask & 10) and $value =~ /^\s*unix\s/)));

    # Is there already such a define?
    $minname= substr($name,0,$minlen);
    for ($i= 0; $i < @deftab; $i++)
    {
    	if ($minlen >= $deftab[$i]->[1] and $minlen <= $deftab[$i]->[2] and
	    substr($deftab[$i]->[0],0,$minlen) eq $minname)
	{
	    if ($name eq $deftab[$i]->[0])
	    {
	    	$deftab[$i]->[3]= $mask;
	    	$deftab[$i]->[4]= $value;
		&{$deftab[$i]->[5]}($name,$value) if $deftab[$i]->[5];
		return;
	    }
	    else
	    {
	    	print "Define $name conflicts with previous define $deftab[$i]->[0]\n";
		return;
	    }
	}
    }

    # Add to definition table;
    push @deftab, [$name, $minlen, $maxlen, $mask, $value];
}


# undefine_one($name)
#   undefine a defined value

sub undefine_one {
    my ($name)= @_;
    my ($i, $len);

    $name=~ s/_//g;
    $len= @deftab;
    for ($i= 0; $i < $len; $i++)
    {
        if ($deftab[$i]->[0] eq $name)
        {
	    $defhash{$name}= undef;
            splice(@deftab, $i, 1);
            last;
        }
    }
}


# undefine_mask($mask)
#   Undefine all values whose mask intersects with the given mask.
#   In particular, undefine_mask(128) erases all conference rc defines,
#   and undefine_mask(64) erases all system rc defines.

sub undefine_mask {
    my ($mask)= @_;
    my ($i);

    for ($i= 0; $i < @deftab; $i++)
    {
        if ($deftab[$i]->[3] & $mask)
        {
	    $defhash{$deftab[$i]->[0]}= undef;
            splice(@deftab, $i, 1);
        }
    }
}


# show_defines()
#    list definitions

sub show_defines {
    my ($row,$val);

    print "What       Is Short For\n";
    foreach $row (@deftab)
    {
	next if $row->[3]&256;
	$val= $row->[4];
	$val=~ s/\n/^J/g;
	$val=~ s/\r/^M/g;
        printf("%-11s%3d %s\n", $row->[0], $row->[3], $val);
    }
}

# expand_cmd_macro($typeref, $cmdref,$argsref,$ok)
#   Given a command keyword and it's argument string, check if it is a macro.
#   If not, do nothing.  If so, do command macro expansion, and return
#   the new command and arguments.  Mask should be
#      1 for OK prompt commands.
#      4 for range argument macros.
#      8 for rfp prompt commands.

sub expand_cmd_macro {
    my ($typeref, $cmdref, $argsref, $mask)= @_;
    my ($row, $len);

    RERUN: while (1)
    {
	$len= length $$cmdref;
	foreach $row (@deftab)
	{
	    if (($mask & $row->[3]) and
	        $len >= $row->[1] and $len <= $row->[2] and
		substr($row->[0], 0, $len) eq $$cmdref)
	    {
		$$argsref= $row->[4].' '.$$argsref;
		($$typeref,$$cmdref)= parse_cmd($argsref,1);
		defined $$typeref or return;
		next RERUN;
	    }
	}
	return;
    }
}

sub cmd_define {
    my ($args)= @_;
    my ($type, $name, $mask, $value);
    ($type, $name)= parse_cmd(\$args,1);
    if (!defined $type)
    {
	show_defines();
	return 1;
    }
    ($type, $mask)= parse_cmd(\$args,0);
    if (!defined $type)
    {
    	undefine_one($name);
	return 1;
    }
    if ($type eq 'S')
    {
    	($type, $value)= parse_cmd(\$args,0);
	if (defined $type)
	{
	    new_define($name,$mask,$value);
	    return 1;
	}
    }
    new_define($name,2,$mask);
    return 1;
}

sub cmd_undef {
    my ($args)= @_;
    my ($type, $name);

    ($type, $name)= parse_cmd(\$args,1);
    if (!defined $type)
    {
	# undefine everything
	undefine_mask(32767);
	return 1;
    }
    if ($name =~ /^\d+$/)
    {
	# undefine everything matching mask
	undefine_mask($name);
	return 1;
    }
    while ($type)
    {
    	undefine_one($name);
	($type, $name)= parse_cmd(\$args,1);
    }
    return 1;
}

1;
